/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.RDSException;

/**
 * &lt;attribut pid="att.tmcOrtsTyp" name="TmcOrtsTyp"&gt; &lt;info&gt;
 * &lt;kurzinfo&gt;Ortstyp und OrtSubtyp des Ortes entsprechend der Festlegungen
 * der LCL.&lt;/kurzinfo&gt; &lt;beschreibung&gt; &lt;absatz&gt;Die Location Sub
 * Types der TIC-Spezifikation werden wie folgt auf den &lt;verweis
 * pid="att.tmcOrtsTyp"/&gt;abgebildet:&lt;/absatz&gt; &lt;absatz&gt;Der Wert
 * des Aufzahlungstyps att.tmcOrtsTyp ist eine sechsstellige Zahl der Form
 * TNNSSS mit T gleich Typ (1=Punkt, 2=Linie, 3=Gebiet), NN gleich Orttypnummer
 * und SSS gleich Untertypnummer entsprechend LCL, jeweils mit führenden
 * Nullen.&lt;/absatz&gt; &lt;absatz&gt;Der TMC-Typ 'Punkt', 'Linie' oder
 * 'Gebiet' ergibt sich aus dem XML Element LocationLocType mit folgender
 * Kodierung:&lt;/absatz&gt; &lt;liste&gt; &lt;listenPunkt&gt;LocationLocType 0:
 * 'Gebiet'&lt;/listenPunkt&gt; &lt;listenPunkt&gt;LocationLocType 1:
 * 'Linie'&lt;/listenPunkt&gt; &lt;listenPunkt&gt;LocationLocType 2:
 * 'Punkt'&lt;/listenPunkt&gt; &lt;/liste&gt; &lt;absatz&gt;Die Orttypnummer
 * ergibt sich aus dem XML Element LocationLocSubType mit folgender
 * Kodierung:&lt;/absatz&gt; &lt;liste&gt; &lt;listenPunkt&gt;LocationLocSubType
 * div 100&lt;/listenPunkt&gt; &lt;/liste&gt; &lt;absatz&gt;Die Untertypnummer
 * ergibt sich aus dem XML Element LocationLocSubType mit folgender
 * Kodierung:&lt;/absatz&gt; &lt;liste&gt; &lt;listenPunkt&gt;LocationLocSubType
 * mod 100&lt;/listenPunkt&gt; &lt;/liste&gt;
 * &lt;absatz&gt;Beispiel:&lt;/absatz&gt; &lt;absatz&gt;LocationLocType = 0
 * -&gt; Typ=3&lt;/absatz&gt; &lt;absatz&gt;LocationLocSubType = 607 -&gt;
 * Orttypnummer=6, Untertypnummer=7&lt;/absatz&gt; &lt;absatz&gt;Wert von
 * att.tmcOrtsTyp: 306007&lt;/absatz&gt; &lt;/beschreibung&gt; &lt;/info&gt;
 * &lt;/attribut&gt;
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class RDSTmcOrtsTyp extends AbstractRDSInfoElement {

	/**
	 * TIC-Definition Ortstypen.
	 *
	 * Wert Typ 0 Area 1 Line 2 Point
	 */
	private static final int[] TIC2DAV = { 3, 2, 1 };

	/**
	 * DAV-Definition Ortstypen.
	 *
	 * Wert Typ 1 Punkt 2 Linie 3 Gebiet
	 */
	private static final int[] DAV2TIC = { -1, 2, 1, 0 };

	/**
	 * TIC Info XML Element: LocationLocType, Type of the location.
	 */
	private static final String XML_ELEMENT_LOCATIONLOCTYPE = "LTP";

	/**
	 * TIC Info XML Element: LocationLocSubType, Subtype of the location.
	 */
	private static final String XML_ELEMENT_LOCATIONLOCSUBTYPE = "LSU";

	/** Typ der Location (TIC). */
	private int locationTyp = 0;

	/** Subtyp der Location (TIC). */
	private int locationSubTyp = 0;

	/**
	 * Erzeugt eine neue Instanz.
	 */
	public RDSTmcOrtsTyp() {
		super(RDSTmcOrtsTyp.XML_ELEMENT_LOCATIONLOCTYPE, "TmcOrtsTyp");
	}

	@Override
	public void einlesenXML(final Node node) throws RDSException {
		if (node.getNodeName().equals(RDSTmcOrtsTyp.XML_ELEMENT_LOCATIONLOCTYPE)) {
			locationTyp = Integer.parseInt(node.getTextContent());
		}

		if (node.getNodeName().equals(RDSTmcOrtsTyp.XML_ELEMENT_LOCATIONLOCSUBTYPE)) {
			locationSubTyp = Integer.parseInt(node.getTextContent());
		}
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) {
		Element element = doc.createElement(RDSTmcOrtsTyp.XML_ELEMENT_LOCATIONLOCTYPE);
		parent.appendChild(element);
		element.setTextContent(String.valueOf(locationTyp));

		element = doc.createElement(RDSTmcOrtsTyp.XML_ELEMENT_LOCATIONLOCSUBTYPE);
		parent.appendChild(element);
		element.setTextContent(String.valueOf(locationSubTyp));

		return element;
	}

	@Override
	public void getDavDaten(final Data davdata) throws RDSException {
		int davtmccode = davdata.getUnscaledValue(getDavAttributName()).intValue();
		locationTyp = getTICLocationTyp(davtmccode / 100000);
		davtmccode %= 10000;
		locationSubTyp = ((davtmccode / 1000) * 100) + (davtmccode % 1000);
	}

	/**
	 * Wandelt einen Locationtyp entsprechend TIC in den Typ entsprechend DaV
	 * um.
	 *
	 * @return Locationtypkennung DaV
	 */
	private int getDavLocationTyp() {
		return RDSTmcOrtsTyp.TIC2DAV[locationTyp];
	}

	/**
	 * Wandelt einen Locationtyp entsprechend DaV in den Typ entsprechend TIC
	 * um.
	 *
	 * @param davtyp
	 *            Locationtyp entsprechend DaV
	 *
	 * @return Locationtypkennung TIC
	 */
	private int getTICLocationTyp(final int davtyp) {
		return RDSTmcOrtsTyp.DAV2TIC[davtyp];
	}

	@Override
	public boolean hatNodeHandler(final Node node) {
		return node.getNodeName().equals(RDSTmcOrtsTyp.XML_ELEMENT_LOCATIONLOCTYPE)
				|| node.getNodeName().equals(RDSTmcOrtsTyp.XML_ELEMENT_LOCATIONLOCSUBTYPE);
	}

	@Override
	public void setDavDaten(final Data davdata) throws RDSException {
		davdata.getUnscaledValue(getDavAttributName())
		.set((getDavLocationTyp() * 100000) + ((locationSubTyp / 100) * 1000) + (locationSubTyp % 100));
	}
}
